@@ -1,6 +1,4 @@
 import os
-import signal
-import glob
 import sys
 
 from .common import debug_print, walk
@@ -8,66 +6,34 @@ from . import base
 from . import uv
 from . import qt
 from . import qta
-
-try:
-    import pyuv
-    debug_print("using pyuv")
-except ImportError:
-    try:
-        import PySide2
-        from PySide2 import QtCore
-        debug_print("using PySide2")
-    except ImportError:
-        try:
-            import PyQt5
-            from PyQt5 import QtCore
-            debug_print("using PyQt5")
-        except ImportError:
-            raise Exception("eventloop needs one of: pyuv or PySide2 or PyQt5 packages to work, none found")
-
-try:
-    import qasync
-    debug_print("using qasync")
-except ImportError:
-    pass
-
-has_pyuv = 'pyuv' in globals()
-has_PySide2 = 'PySide2' in globals()
-has_PyQt5 = 'PyQt5' in globals()
-has_qasync = 'qasync' in globals()
-
-""" template
-if has_pyuv:
-    pass
-elif has_PySide2 or has_PyQt5:
-    pass
-"""
+from .common import flavour, FLAVOUR_NONE, FLAVOUR_PYUV, FLAVOUR_PYSIDE2, FLAVOUR_QT5, FLAVOUR_PYSIDE2_QASYNC, FLAVOUR_QT5_QASYNC
 
 def EventLoop(app = None):
-    if has_pyuv:
-        return uv.EventLoop()
-    elif has_PySide2 or has_PyQt5:
-        if has_qasync:
-            return qta.EventLoop(app)
-        else:    
-            return qt.EventLoop(app)
+    return {
+        FLAVOUR_NONE: lambda app: None,
+        FLAVOUR_PYUV: lambda app: uv.EventLoop(),
+        FLAVOUR_PYSIDE2: lambda app: qt.EventLoop(app),
+        FLAVOUR_QT5: lambda app: qt.EventLoop(app),
+        FLAVOUR_PYSIDE2_QASYNC: lambda app: qta.EventLoop(app),
+        FLAVOUR_QT5_QASYNC: lambda app: qta.EventLoop(app),
+    }[flavour](app)
 
-def FileSystemWatch():
-    if has_pyuv:
-        return uv.FileSystemWatch()
-    elif has_PySide2 or has_PyQt5:
-        return qt.FileSystemWatch()
+def FileSystemWatch(loop):
+    if flavour == FLAVOUR_PYUV:
+        return uv.FileSystemWatch(loop)
+    else:
+        return qt.FileSystemWatch(loop)
 
 def SingleShotTimer():
-    if has_pyuv:
+    if flavour == FLAVOUR_PYUV:
         return uv.SingleShotTimer()
-    elif has_PySide2 or has_PyQt5:
+    else:
         return qt.SingleShotTimer()
 
 def Timer():
-    if has_pyuv:
+    if flavour == FLAVOUR_PYUV:
         return uv.Timer()
-    elif has_PySide2 or has_PyQt5:
+    else:
         return qt.Timer()
 
 class Schedule:
@@ -111,7 +77,7 @@ class Schedule:
         timer.start(timeout, self.on_timeout)
         self._timer = timer
 
-def on_file_changed(path, include=None, exclude=None, timeout=1, loop=None, recursive=True):
+def on_file_changed(path, include=None, exclude=None, timeout=1, loop=None, recursive=True, terminate_after=None):
 
     def decorator(func):
 
@@ -125,16 +91,22 @@ def on_file_changed(path, include=None, exclude=None, timeout=1, loop=None, recu
 
         executor = base.FuncExecutor(func)
         
-        watch = FileSystemWatch()
+        watch = FileSystemWatch(loop_)
         schedule = Schedule(executor)
         watch.start(path, on_change, recursive=recursive, include=include, exclude=exclude)
 
+        terminate_timer = None
+        if terminate_after is not None:
+            terminate_timer = SingleShotTimer()
+            terminate_timer.start(terminate_after, loop_.stop)
+
         if loop is None:
             loop_.start()
-        else:
-            loop._handles.append(watch)
-            loop._handles.append(schedule)
-            loop._handles.append(executor)
+        
+        loop_._handles.append(watch)
+        loop_._handles.append(schedule)
+        loop_._handles.append(executor)
+        loop_._handles.append(terminate_timer)
 
         return func
     
