@@ -1,23 +1,23 @@
 from . import base
 import signal
-from .common import walk, EVENT_CHANGE, debug_print, path_matches
+from .common import debug_print, walk, EVENT_CHANGE, debug_print, path_matches
 import glob
 import os
+from .common import flavour, FLAVOUR_NONE, FLAVOUR_PYUV, FLAVOUR_PYSIDE2, FLAVOUR_QT5, FLAVOUR_PYSIDE2_QASYNC, FLAVOUR_QT5_QASYNC
 
-try:
-    import PySide2
+if flavour in [FLAVOUR_PYSIDE2, FLAVOUR_PYSIDE2_QASYNC]:
     from PySide2 import QtCore
-except ImportError:
-    try:
-        import PyQt5
-        from PyQt5 import QtCore
-    except ImportError:
-        pass
+elif flavour in [FLAVOUR_QT5, FLAVOUR_QT5_QASYNC]:
+    from PyQt5 import QtCore
 
 class EventLoop(base.EventLoop):
 
     def __init__(self, app = None):
         super().__init__()
+        self._watchers = []
+        debug_print("qt.EventLoop")
+        if app is None:
+            app = QtCore.QCoreApplication.instance()
         if app is None:
             app = QtCore.QCoreApplication([])
         self._app = app
@@ -25,9 +25,25 @@ class EventLoop(base.EventLoop):
     def start(self):
         signal.signal(signal.SIGINT, signal.SIG_DFL)
         self._app.exec_()
+        debug_print("EventLoop started")
+
+    def addWatcher(self, watcher):
+        self._watchers.append(watcher)
         
+    def _cleanWatchers(self):
+        watcher: QtCore.QFileSystemWatcher
+        for watcher in self._watchers:
+            paths = watcher.files() + watcher.directories()
+            if len(paths):
+                watcher.removePaths(paths)
+        self._watchers = []
+
     def stop(self):
-        QtCore.QCoreApplication.quit()
+        self._cleanWatchers()
+        if self._app:
+            self._app.exit()
+            self._app = None
+            debug_print("EventLoop stoped")
 
 class Timer(base.Timer):
 
@@ -54,10 +70,15 @@ class SingleShotTimer(Timer):
 
 class FileSystemWatch(base.FileSystemWatch):
 
+    def __init__(self, loop):
+        super().__init__()
+        self._loop = loop
+
     def start(self, path, callback, include = None, exclude = None, recursive = False):
         super().start(path, callback, include, exclude, recursive)
         watcher = QtCore.QFileSystemWatcher()
         self._watch = watcher
+        self._loop.addWatcher(watcher)
 
         if not recursive:
             if glob.has_magic(path):
